use crate::resources::flashing_message::FlashingMessage;
use crate::resources::{Map, Resources};
use crate::{graphics, input, levels, systems};
use bracket_lib::prelude::*;
use hecs::World;

pub struct State {
    world: World,
    resources: Resources,
}

impl GameState for State {
    fn tick(&mut self, bterm: &mut BTerm) {
        input::handle(&mut self.world, &mut self.resources, bterm);
        systems::run(&mut self.world, &mut self.resources);
        graphics::draw(&self.world, &mut self.resources, bterm);

        if self.resources.should_advance_level {
            self.next_level();
        }
    }
}

impl State {
    pub fn new(world: World, resources: Resources) -> Self {
        State { world, resources }
    }

    fn next_level(&mut self) {
        self.resources.level_number += 1;

        self.world.clear();
        self.resources.map = Map::from(levels::get_level(self.resources.level_number));
        self.resources.map.spawn_entities(&mut self.world);

        self.resources.flashing_message =
            Some(FlashingMessage::from("Press any key to begin this level."));

        self.resources.should_advance_level = false;
    }
}
